<?PHP
/**
 * PHP Version 5.x
 * Copyright (C) 2010 All Rights Reserved.
 *
 * @Application : Cache.class.php
 * @Encode : 65001 Unicode ( UTF-8 NO BOM )
 * @Description : 变量缓存类, 全局变量的替代方法
 * @Author : HK <hkyyang@qq.com>
 * @Create : 2010/12/4
 */
class Cache {

	// 静态变量缓存
	static public $_cache = array();
	// 自增数组替换符
	static public $_auto = '*';
	// 赋值运算模板
	static public $operators = array(
		// Numeric
		'+=' => '+= $value',
		'-=' => '-= $value',
		'=-' => '= $value - $last_value',
		'*=' => '*= $value',
		'/=' => '= $last_value == 0 ? 0 : ( $value != 0 ? $last_value / $value : $last_value )',
		'=/' => '= $value == 0 ? 0 : ( $last_value != 0 ? $value / $last_value : $value )',
		'%=' => '= $value == 0 ? 0 : $last_value % $value',
		// String
		'.=' => '.= $value',
		'=.' => '= $value . $last_value'
	);


	/**
	 * set 方法，设置变量缓存
	 * Array 不被处理为普通非标量，而是算做多维赋值，并且数组内键名大写化
	 * 支持以"." 构造的N级键名
	 * 支持任何类型及非标量作为值
	 * 运算符只在值为标量(除Boolean)并且有原值时有效, 否则会产生不可预料的结果
	 *
	 * @static
	 * @access public
	 * @param String | Array, $name 变量名称、键名构造或配置数组
	 * @param String | Null, $value 变量值
	 * @param String | Null, $operator 运算符, 必须为赋值运算
	 * @return *
	 */
	static public function set( $name = null, $value = null, $operator = null ) {

		if( is_null( $name ) || is_bool( $name ) ) return null;
		// 统一大写
		if( is_array( $name ) ) {
			$name = array_change_key_case( $name, CASE_UPPER );
		} else {
			$name = strtoupper( $name );
		}
		// 参数一为数组时，忽略参数二
		if( is_array( $name ) ) {
			return self :: $_cache = array_merge( self :: $_cache, $name );
		} else {
			$_temp = explode( '.', $name );
			$_key = '';
			// 构造配置多维数组
			for( $i = 0, $n = count( $_temp ); $i < $n; $i ++ ) {
				// 自增数组
				if( $_temp[ $i ] == self :: $_auto ) {
					$is_auto = true;
					$_key .= "[]";
				} else {
					$_key .= "['" . $_temp[$i] . "']";
				}
			}
			// 当有运算符并有上次赋值时
			if( array_key_exists( $operator, self :: $operators ) ) {
				$last_value = eval( "return isset( self :: \$_cache$_key ) ? self :: \$_cache$_key : null;" );
				if( ! is_null( $last_value ) ) {
					$oper = self :: $operators[ $operator ];
					return eval( "return self :: \$_cache$_key $oper;" );
				}
			}
			// 对非标量支持
			return eval( "return self :: \$_cache$_key = \$value;" );
		}

	}

	/**
	 * get 方法，获取变量值
	 *
	 * @static
	 * @access public
	 * @param String , $name 变量名称或键名构造
	 * @return *
	 */
	static public function get( $name = null ) {
		if( is_null( $name ) ) return self :: $_cache;
		$_temp = explode( '.', strtoupper( $name ) );
		$_key = '';
		for( $i = 0, $n = count( $_temp ); $i < $n; $i ++ ) {
			$_key .= "['" . $_temp[$i] . "']";
		}
		return eval( "return isset( self :: \$_cache$_key ) ? self :: \$_cache$_key : null;" );
	}

	/**
	 * search_cache 方法，尝试从已有缓存中搜索一个值的键名或索引构造字符串
	 * 搜索值可以是一个非标量但不包含数组和null
	 * 顺序递归N维深度的搜索
	 *
	 * @static
	 * @access public
	 * @param ! Array, $value 值, 可以是对象或是资源类型
	 * @param String, $parent 键名构造字符串,指定在这个范围内搜索
	 * @param Boolean : Default False, $strict 严格匹配，包括类型
	 * @return Null | String
	 */
	static public function search_cache( $value, $parent = null, $strict = false ) {

		// 当指定范围时不存在时返回null
		if( is_null( self :: get( $parent ) ) || is_array( $value ) || is_null( $value ) ) {
			return null;
		}
		$c = self :: get( $parent );
		if( is_array( $c ) ) {
			foreach( $c as $k => $v ) {
				$key = $parent ? $parent . '.' . $k : $k;
				$tmp = self :: in_cache( $value, $key, $strict );
				if( is_null( $tmp ) ) continue;
				return $tmp;
			}
		} else {
			if( $strict == true ) {
				if( $value === $c ) return $parent;
			} else if( $value == $c ) {
				return $parent;
			}
		}
		return null;
	}


}
?>