<?PHP

/**
 * 创建一个指定大小的文件
 * 通常用作占用磁盘空间的临时文件
 * 在低配路由上，大文件创建效率极低，
 *
 * @param string $file 文件路径
 * @param integer $byte 大小
 * @param Boolean $init 初始创建
 * @return Boolean
 */
function createBytes( $file, $bytes, $init = false ) {
	clearstatcache();
	// 当同名文件存在，并且大小相同，不做任何处理
	if( file_exists( $file ) && filesize( $file ) == $bytes ) {
		return true;
	}
	$handle = fopen( $file, 'wb' );
	// 移动指针到指定大小-1位置
	fseek( $handle, $bytes - 1 );
	fwrite( $handle, ' ' );
	if ( ! fclose( $handle ) ) {
		// 创建失败，删除
		if ( file_exists( Cache :: get( 'TASK.TEMP_FILE' ) ) ) {
			unlink( Cache :: get( 'TASK.TEMP_FILE' ) );
		}
		log_write( '无法创建临时文件，可能是磁盘空间不足或目录设置错误！' );
		return false;
	}
	return true;
}

// 检查每个线程缓存数据大小，2个不为空时返回TRUE
function thread_data_check( $CACHE ) {

	if( ! is_array( $CACHE ) || count( $CACHE ) < 1 ) return false;
	$not_null = 0;
	foreach( $CACHE as $th ) {
		if( empty( $th[ '_BYTES' ] ) || $th[ '_BYTES' ] < 1 ) continue;
		$not_null ++;
	}

	if( $not_null >= 2 ) return true;

	return false;
}

// 对比有内容的缓存的同一长度内容，只要有一个线程和头线程前100字节相等返回true
function checkCache( $CACHE, $bytes = 100 ) {
	$temp = null;
	foreach( $CACHE as $th ) {
		if( empty( $th[ 'DATA' ] ) ) continue;
		if( $temp == null ) {
			$temp = substr( $th[ 'DATA' ], 0, $bytes );
			continue;
		}
		// 二进制比较
		if( strcmp( $temp, substr( $th[ 'DATA' ], 0, $bytes ) ) == 0 ) {
			return true;
		}
	}
	return false;
}

/**
 * 格式化文件大小
 *
 * @param integer $bytes 字节大小
 * @return String
 */
function sizeCount( $bytes ) {
	if ( $bytes > 1073741824 ) {
		$bytes = round( $bytes / 1073741824 * 100 ) / 100;
		return array( 'size' => $bytes, 'full_name' => 'Gib', 'name' => 'GB', 'short' => 'G' );
	}else if ( $bytes > 1048576 ) {
		$bytes = round( $bytes / 1048576 * 100 ) / 100;
		return array( 'size' => $bytes, 'full_name' => 'Mib', 'name' => 'MB', 'short' => 'M' );
	}else if ( $bytes > 1024 ) {
		$bytes = round( $bytes / 1024 * 100 ) / 100;
		return array( 'size' => $bytes, 'full_name' => 'Kib', 'name' => 'KB', 'short' => 'K' );
	}else {
		$bytes = $bytes;
		return array( 'size' => $bytes, 'full_name' => 'B', 'name' => 'B', 'short' => 'Bytes' );
	}
}

/**
 * 大小转换
 *
 * @param integer $num 大小数字
 * @param integer $inUnit 当前大小单位
 * @param integer $outUnit 输出大小单位
 * @param integer $precision 小数点保留位数
 * @return Integer
 */
function sizeFormat( $num , $inUnit, $outUnit, $precision = 0 ) {
	if ( $num < 1 ) return 0;

	$inUnit = strtoupper( substr( $inUnit, 0, 1 ) );
	$outUnit = strtoupper( substr( $outUnit, 0, 1 ) );
	switch ( $inUnit ) {
		case 'G' : $num *= 1073741824;
			break;
		case 'M' : $num *= 1048576;
			break;
		case 'K' : $num *= 1024;
			break;
		case 'B' : break;
		default : return false;
	}
	$result = 0;
	switch ( $outUnit ) {
		case 'G' : return round( $num / 1073741824 * 100, $precision ) / 100;
		case 'M' : return round( $num / 1048576 * 100, $precision ) / 100;
		case 'K' : return round( $num / 1024 * 100, $precision ) / 100;
		case 'B' : return $num;
	}
}

// 判断AJAX请求
function isAjax() {
	if ( isset( $_SERVER[ 'HTTP_X_REQUESTED_WITH' ] ) ) {
		if ( 'xmlhttprequest' == strtolower( $_SERVER[ 'HTTP_X_REQUESTED_WITH' ] ) ) {
			return true;
		}
	}
	return false;
}

// 加载配置
function cache_load_config( $tb ) {
	$config = Cache :: get( 'SQLITE' ) -> table( $tb ) -> select();
	$cache = array();
	foreach( $config as $val ) {
		$cache[ $val[ 'name' ] ] = $val[ 'value' ];
	}
	return $cache;
}

function parseURL( $url ) {

	if( substr( $url, 0, 8 ) == 'thunder:' ) {
		return substr( base64_decode( substr( $url, 10, -2 ) ), 2 );
	}

	return $url;
}

function log_write( $str ) {
	Cache :: get( 'SQLITE' ) -> table( 'logs' ) -> replace( array(
		'text' => $str,
		'create_time' => time()
	) );
	return true;
}

?>