( function( $ ) {

	/**
	 * 弹出层窗口
	 * 动画效果：水平居中、垂直居中高度2/3位置向垂直居中位置淡入
	 *						水平居中、垂直居中高度+1/3位置向下淡出
	 *
	 * @param String title 标题
	 * @param String content 内容
	 * @param Object callback 回调函数, 出现按钮, 确定执行函数
	 */
	$.Dialog = function( title, content, callback, closeCallback ) {
		$( '#Dialog-Handler' ).html( title + '<span id="Dialog-Handler-Close">x</span>' );
		if( content != null ) $( '#Dialog-Message' ).html( content );
		$( '#Dialog-Button button' ).unbind();
		$( '#Dialog-Button button' ).show();
		if( callback === false ) {
			$( '#Dialog-Button' ).hide();
		} else if( $.isFunction( callback ) ) {
			$( '#Dialog-Button button[ac="confirm"]' ).bind( 'click', function() {
				if( $( this ).attr( 'class' ).indexOf( 'disabled' ) > -1 ) return false;
				callback();
				return false;
			} );
			$( '#Dialog-Button button[ac="confirm"]' ).show();
		} else {
			$( '#Dialog-Button button[ac="cancel"]' ).hide();
			$( '#Dialog-Button button[ac="confirm"]' ).bind( 'click', function() {
				if( $( this ).attr( 'class' ).indexOf( 'disabled' ) > -1 ) return false;
				$.DialogClose();
				if( $.isFunction( closeCallback ) ) closeCallback();
			} );
		}

		$( '#Dialog-Button button[ac="cancel"]' ).bind( 'click', function() {
			if( $( this ).attr( 'class' ).indexOf( 'disabled' ) > -1 ) return false;
			$.DialogClose();
			if( $.isFunction( closeCallback ) ) closeCallback();
		} );
		$( '#Dialog-Handler-Close' ).mouseover( function() {
			$( this ).css( 'color', '#FFF' );
		} ).mouseout( function() {
			$( this ).css( 'color', '#CCC' );
		} ).bind( 'click', function() {
			$.DialogClose();
			if( $.isFunction( closeCallback ) ) closeCallback();
		} );

		// 未结束调用
		if( $DIALOG.DISPLAY == true ) {
			var PS = $( '#Dialog' ).moveCenter( null, true );
			$( '#Dialog' ).moveTo( PS.left, PS.top, 350, function() {
				$.DialogFlash( 8 );
			} );
		} else {
			$.DialogShow();
			$DIALOG.DISPLAY = true;
		}

		return false;
	};

	// 闪烁效果
	$.DialogFlash = function( fn ) {
		if( fn < 1 ) return;
		if( $DIALOG.ANIMATE != null ) clearTimeout( $DIALOG.ANIMATE );
		if( fn % 2 == 0 ) {
			$( '#Dialog-Handler' ).css( 'background', '#CCCCCC' );
		} else {
			$( '#Dialog-Handler' ).css( 'background', '#595A52' );
		}
		fn --;
		$DIALOG.ANIMATE = setTimeout( function() {
			$.DialogFlash( fn );
		}, 100 );
	};

	$.DialogClose = function( fn ) {
		if( fn == true ) {
			$( '#Dialog' ).hide();
			$DIALOG.DISPLAY = false;
		} else {
			var PS = $( '#Dialog' ).offset();
			var top = PS.top > 0 ? + Math.round( PS.top / 3 ) * 2 : 0;
			top += PS.top;
			$( '#Dialog' ).animate( { 'top' : top + 'px', 'opacity' : 0 }, 'normal', function() {
				$( this ).css( 'display', 'none' );
				$DIALOG.DISPLAY = false;
			} );
		}
		$( '#Mask' ).hide();
		return false;
	};

	$.DialogShow = function() {
		var PS = $( '#Dialog' ).css( { 'top' : 0, 'opacity' : 0, 'display' : 'block' } ).moveCenter( null, true );
		var top = PS.top > 0 ? Math.floor( PS.top / 3 ) * 2 : 0;
		$( '#Dialog' ).css( { 'left' : PS.left + 'px', 'top' : top + 'px' } ).animate( { 'top' : PS.top + 'px', 'opacity' : 1.0 }, 'normal' );
	};

	/**
	 * Unix TimeStamp 格式化
	 *
	 * @param Integer timeStamp 时间戳
	 * 例 YYYY-MM-dd hh:mm:ss
	 * @param String format 格式
	 * @return String
	 */
	$.dateFormat = function( timeStamp, format ) {
		var d = timeStamp != undefined ? new Date(  parseInt( timeStamp )  * 1000 ) : new Date();
		var o = {
			"M+" :  d.getMonth()+1,  //month
			"d+" :  d.getDate(),     //day
			"h+" :  d.getHours(),    //hour
			"m+" :  d.getMinutes(),  //minute
			"s+" :  d.getSeconds(), //second
			"q+" :  Math.floor( ( d.getMonth()+3 ) / 3 ),  //quarter
			"S"  :  d.getMilliseconds() //millisecond
		}

		if( /(Y+)/.test( format ) ) {
			format = format.replace( RegExp.$1, ( d.getFullYear()+"" ).substr( 4 - RegExp.$1.length ) );
		}

		for( var k in o ) {
			if( new RegExp( "("+ k +")" ).test( format ) ) {
				format = format.replace( RegExp.$1, RegExp.$1.length==1 ? o[k] : ( "00"+ o[k] ).substr( ( ""+ o[k] ).length ) );
			}
		}
		return format;
	};

	// Cookie
	$.cookie = function( name, value, options ) {
		if ( typeof value != 'undefined' ) { // name and value given, set cookie
			options = options || {};
			if ( value === null ) {
				value = '';
				options.expires = -1;
			}
			var expires = '';
			if( options.expires && ( typeof options.expires == 'number' || options.expires.toUTCString ) ) {
				var date;
				if ( typeof options.expires == 'number' ) {
					date = new Date();
					date.setTime( date.getTime() + ( options.expires * 24 * 60 * 60 * 1000 ) );
				} else {
					date = options.expires;
				}
				expires = '; expires=' + date.toUTCString(); // use expires attribute, max-age is not supported by IE
			}
			var path = options.path ? '; path=' + ( options.path ) : '';
			var domain = options.domain ? '; domain=' + ( options.domain ) : '';
			var secure = options.secure ? '; secure' : '';
			document.cookie = [ name, '=', encodeURIComponent( value ), expires, path, domain, secure ].join( '' );
		} else { // only name given, get cookie
			var cookieValue = null;
			if( document.cookie && document.cookie != '' ) {
				var cookies = document.cookie.split( ';' );
				for( var i = 0; i < cookies.length; i++ ) {
					var cookie = jQuery.trim( cookies[ i ] );
					if ( cookie.substring( 0, name.length + 1 ) == ( name + '=' ) ) {
						cookieValue = decodeURIComponent( cookie.substring( name.length + 1 ) );
						break;
					}
				}
			}
			return cookieValue;
		}
	};

} ) ( jQuery );