/**
 *
 * @Application : Event.func.js
 * @Encode : 65001 Unicode ( UTF-8 NO BOM )
 * @Description : 事件处理函数
 * @Author : HK <hkyyang@qq.com>
 * @Update : 2010/12/18
 */
/* ---------------------------- GLOBAL VARIABLE ---------------------------- */

var $SETTING = {
	REFRESH : 3000
};

var $TASK = {
	DATA : null,
	ACTIVE : {}, // 活动的任务
		/* ---------------------------------------------------
		ID( OBJECT ) : {
			RUNTIME : 0,
			BYTES : 0,
			PROGRESS : null,
			TIME_STAMP : 0,
			INTERVAL_ID : null
		}
		--------------------------------------------------- */
	LAST_FINISH : {} // 活动任务完成转换
		/* ---------------------------------------------------
		ID( OBJECT ) : {
			PROGRESS : null,
			RATE : null
		}
		--------------------------------------------------- */
};

var $DIALOG = {
	DISPLAY : false,
	ANIMATE : null
};

var $FILTER = {
	DISPLAY : false,
	CHANGE : false
};

var $TASKLIST = {
	MOUSEON : false, // 任务列表鼠标悬停时
	SELECTED : null // 已经选中的任务列ID
};

var $SELECT = {
	OPEN : false,
	MOUSEON : false,
	SELECTED : null,
	OBJ : null
};

var $TITLECONTENTS = false;
var $INFO_RIGHT = {
	OPEN : false,
	MOUSEON : false
};
/* ---------------------------- FUNCTIONS ---------------------------- */

// 自适应格式化一个大小(字节)
function sizeCount( bytes ) {
	if ( bytes >= 1073741824 ) {
		return round( bytes / 1073741824, 2 ) + ' GB';
	} else if ( bytes >= 1048576 ) {
		return round( bytes / 1048576, 2 ) + ' MB';
	} else if ( bytes >= 1024 ) {
		return round( bytes / 1024, 2 ) + ' KB';
	}
	return round( bytes ) + ' Bytes';
}

// 计算进度百分比
function rateCount( a, b ) {
	if( a < 1 || b < 1 ) return 0;
	return round( a / b * 100, 2 );
}

// 格式化秒数到一个时间表示( 天(如果大于24小时的话)、时、分、秒)
function timeCount( second ) {
	function timeFormat( s ) {
		if( s < 10 ) return '0' + s;
		return s;
	}
	var day, hour, minite;
	minite = second >= 60 ? Math.floor( second / 60 ) : 0;
	hour = minite >= 60 ? Math.floor( minite / 60 ) : 0;
	day = hour >= 24 ? Math.floor( hour / 24 ) : 0;
	second = second >= 60 ? timeFormat( second % 60 ) : timeFormat( second );
	if( day > 0 ) return day + ' Day ' + timeFormat( hour ) + ':' + timeFormat( minite ) + ':' + second ;
	else return timeFormat( hour ) + ':' + timeFormat( minite ) + ':' + second;
}

// 优化的Math.round ，可定义保留浮点位数
function round( v, e ) {
	if( e == null || e < 1 ) return Math.round( v );
	for( var t = 1 ; e > 0; t *= 10, e -- );
	return Math.round( v * t ) / t;
}

// 获取本地时间戳,长度13
function timeStamp() {
	return Date.parse( new Date() );
}

// 创建一个自制的SELECT标签
function createSelectUI( id, options, selected, style ) {
	var html = '<div id="'+ id +'" class="select" style="' + style + '"><ul>';
	for( var value in options ) {
		if( selected == value ) {
			html += '<li value="' + value + '" class="selected"><div class="pic"></div><div class="text">' + options[ value ]  + '</div></li>';
		} else {
			html += '<li value="' + value + '"><div class="pic"></div><div class="text">' + options[ value ]  + '</div></li>';
		}
	}
	html += '</ul></div>';
	return html;
}
// 从自制的SELECT里获取selected的值
function getSelectValue( id ) {
	return $( '#' + id + ' ul li:eq(0)' ).attr( 'value' );
}

function infoRightShow( id ) {
	$INFO_RIGHT.OPEN = true;
	$( '#map div' ).hide();
	var info = getTask( id );
	if( info == null ) return;
	setRightData( info );
	switch( info.status ) {
		case "1" :
			break;
		case "2" :
			break;
		case "3" :
			break;
		case "4" :
			break;
		case "5" :
			$( '#map .complete' ).show();
			break;
		case "6" :
			break;
	}
	var w = $( document ).width() - 304;
	$( '#Task-list' ).css( 'width', w );
	$( '#infoRight' ).css( { 'width' : '300px', 'border-left' : '1px solid #CCC', 'display' : 'block' } );
}

function setRightData( info ) {
	var name = info.name;
	if( name.length >= 35 ) {
		name = name.substr( 0, 35 ) + '...' ;
	}
	$( '#info .name' ).html( name );
	$( '#info .name' ).attr( 'titleContents', info.name );
	$( '#info .size' ).html( sizeCount( info.size ) );
	$( '#info .set_thread' ).html( info.set_thread );
	$( '#info .thread' ).html( info.set_thread );
	$( '#info .create_time' ).html( $.dateFormat( info.create_time, 'YYYY-MM-dd hh:mm:ss' ) );
	$( '#info .runtime' ).html( timeCount( info.runtime ) + ' (' + info.runtime + ' 秒)' );
	if( info.data > 0 ) {
		$( '#info .speed' ).html( sizeCount( info.data / info.runtime ) + '/s' );
	} else {
		$( '#info .speed' ).html( 'N/A' );
	}
	if( info.status == 5 ) {
		$( '#info .complete_time' ).html( $.dateFormat( info.update_time, 'YYYY-MM-dd hh:mm:ss' ) );
	} else {
		$( '#info .complete_time' ).html( 'N/A' );
	}
}

function infoRightHide( id ) {
	$INFO_RIGHT.OPEN = false;
	$( '#infoRight' ).css( { 'width' : 0, 'display' : 'none' });
	$( '#Task-list' ).css( 'width', '100%' );
}

/*
 * 数字增长的动画效果
 * 目前效果不是很好，待完善
 */
function numplus( n, max, obj, ext ) {
	var speed = 15;
	if( n > max ) return;
	var fnum1 = n.toString().substr( n.toString().indexOf( '.' ) + 1 ).length;
	var fnum2 = max.toString().substr( max.toString().indexOf( '.' ) + 1 ).length;
	if( max - n > 9 && n + 10 <= max ) n += 10;
	if( max - n > 0.9 && n + 1 <= max ) n += 1;
	if( fnum1 >= 1 || fnum2 >= 1 ) n += 0.1;
	if( fnum1 >= 2 || fnum2 >= 2 ) n += 0.01;
	if( n > max ) n = max;
	obj.html( round( n, 2 ) + ext );
	if( n >= max ) return;
	setTimeout( function() {
		numplus( n , max, obj, ext );
	}, speed );
}

/* ---------------------------- EVENTS ---------------------------- */

// 顶部按钮事件
function topButtonsEvents() {

		switch( $( this ).attr( 'id' ) ) {
			case 'info-link' :
				$.Dialog( '关于', 'Multi-Get 1.1<br>基于PHP的HTTP多线程下载工具<br>2010/12 功能基本完善了...<br> hkyyang@qq.com' );
				break;
			case 'filter-link' :
				if( $FILTER.CHANGE == true ) return;
				$FILTER.CHANGE = true;
				if( $FILTER.DISPLAY == true ) {
					$( '#filter-bar [view=all]' ).click();
					$( '#filter-bar' ).animate( { 'marginTop':'-24px' }, 'normal', function() {
						$FILTER.DISPLAY = false;
						$FILTER.CHANGE = false;
					} );
					var task_h = $( '#Task-list' ).height() - 24;
					$( '#Task-list' ).animate( { 'height' : task_h } );
				} else {
					$( '#filter-bar [view=all]' ).click();
					$( '#filter-bar' ).animate( { 'marginTop':'0px' }, 'normal', function() {
						$FILTER.DISPLAY = true;
						$FILTER.CHANGE = false;
					} );
					var task_h = $( '#Task-list' ).height() + 24;
					$( '#Task-list' ).animate( { 'height' : task_h } );
				}
				break;
			case 'new-link' :
				$( '#Mask' ).show();
				var selectDOM = createSelectUI( 'task-status', { '1' : '立即下载', '2' : '队列等待', '4' : '手动下载' }, '1', 'left:90px; top:123px;' );
				$.Dialog( '新建任务', '<textarea id="newTask"></textarea><div style="line-height:30px;">线程数<input id="thread-input" value="5" maxlength="1">' + selectDOM + '</div><div id="newTaskErr"></div>', function() {
					var url = $.trim( $( '#newTask' ).val() );
					if( url.length < 1 ) {
						$( '#newTaskErr' ).html( '请输入链接地址!' );
					} else if( ! url.match(/http:\/\/.+/) ) {
						$( '#newTaskErr' ).html( '链接地址无效，仅支持HTTP协议，请重新输入。' );
					} else {
						$.DialogClose( true );
						$( '#Mask' ).hide();
						$( '#new-link' ).addClass( 'disabled' );
						$.post( 'main.php?action=create&thread_num=' + $( '#thread-input' ).val() + '&task_status=' + getSelectValue( 'task-status' ), { 'task_url' : encodeURI( url ) }, function(x){
							if( x != null && x.error != null ) {
								switch( x.error ) {
									case '1' : $.Dialog( '错误信息', '链接地址无效，或者不是一个正常的可下载文件，<br>仅支持HTTP协议，请重新输入。' ); break;
									case '2' : $.Dialog( '警告', '此任务不支持断点续传和多线程，强制使用单线程下载。' ); break;
									case '3' : $.Dialog( '警告', '此任务已经存在，如果想重新下载，请先删除原任务。' ); break;
								}
							}
							$( '#new-link' ).removeClass( 'disabled' );
						}, 'json' );
					}
				} );
				break;
			case 'remove-link' :
				if( $TASKLIST.SELECTED == null ) return;
				var selectDOM = createSelectUI( 'remove-type', { '1' : '同时删除文件', '2' : '保留已下载文件' }, '1', 'left:10px; top:88px;' );
				var info = getTask( $TASKLIST.SELECTED );
				if( info == null ) return;
				$( '#Mask' ).show();
				$.Dialog( '删除任务', '<div>'+ info.name +'</div><div>'+ sizeCount( info.size ) +'</div>' + selectDOM, function() {
					$( '#Mask' ).hide();
					clearActive( info.id );
					$.DialogClose( true );
					$.get( 'main.php?action=remove&id=' + info.id + '&type='+ getSelectValue( 'remove-type' ) +'&rand=' + Math.random() );
				}, function() { $( '#Mask' ).hide(); } );

				break;
			case 'pause-link' :
				if( $TASKLIST.SELECTED == null ) return;
				$( this ).addClass( 'disabled' );
				var temp_val = $TASKLIST.SELECTED;
				$( '[task_id=' + temp_val + ']' ).attr( 'status', 4 ).find( '.status' ).css( 'backgroundPosition', 'left -48px' );
				setTimeout( function() { clearInterval( $TASK.ACTIVE[ temp_val ].INTERVAL_ID ); }, $SETTING.REFRESH / 2 );
				$.get( 'main.php?action=pause&id=' + temp_val + '&rand=' + Math.random() );
				break;
			case 'resume-link' :
				if( $TASKLIST.SELECTED == null ) return;
				$( this ).addClass( 'disabled' );
				$( '[task_id=' + $TASKLIST.SELECTED + ']' ).attr( { 'status' : 2 } ).find( '.status' ).css( 'backgroundPosition', 'left -24px' );
				$.get( 'main.php?action=resume&id=' + $TASKLIST.SELECTED + '&rand=' + Math.random() );
				break;
			case 'pause-all-link' :
				$( this ).addClass( 'disabled' );
				$.get( 'main.php?action=pause_all&rand=' + Math.random() );
				break;
			case 'resume-all-link' :
				$( this ).addClass( 'disabled' );
				$.get( 'main.php?action=resume_all&rand=' + Math.random() );
				break;
			case 'clear-db-link' :
				$.Dialog( '清空数据库，慎用！', '即将清除所有任务，但不包含已经下载完成的文件。<br>是否继续?', function() { $.DialogClose(); $.get( 'main.php?action=truncate' ); } );
				break;
		}
}

// 过滤按钮事件
function filterButtonsEvents() {
	if( $( this ).attr( 'class' ).indexOf( 'selected' ) > -1 ) return false;
	$( '#filter-bar li' ).removeClass( 'selected' );
	$( this ).addClass( 'selected' );
	// 显示过滤内容
	switch( $( this ).attr( 'view' ) ) {
		case 'all' : $( '#Task-list>div' ).show(); break;
		case 'download' : $( '#Task-list>div' ).hide(); $( '#start-list' ).show(); break;
		case 'standby' : $( '#Task-list>div' ).hide(); $( '#standby-list' ).show(); break;
		case 'paused' : $( '#Task-list>div' ).hide(); $( '#pause-list' ).show(); break;
		case 'complete' : $( '#Task-list>div' ).hide(); $( '#complete-list' ).show(); break;
		case 'failure' : $( '#Task-list>div' ).hide(); $( '#failure-list' ).show(); break;
	}
}

// @LIVE 任务列表事件，以任务的状态决定开放的按钮
function taskListEvents() {

	$TASKLIST.SELECTED = $( this ).attr( 'task_id' );
	$( '#Task-list [task_id]' ).removeClass( 'selected' );
	$( this ).addClass( 'selected' );

	switch( $( this ).attr( 'status' ) ) {
		case '2' : // STANDBY
			infoRightShow( $TASKLIST.SELECTED );
			$( '#resume-link' ).removeClass( 'disabled' );
			$( '#remove-link' ).removeClass( 'disabled' );
			break;
		case '1' : // INIT
		case '3' : // START
			infoRightShow( $TASKLIST.SELECTED );
			$( '#pause-link' ).removeClass( 'disabled' );
			$( '#remove-link' ).removeClass( 'disabled' );
			break;
		case '4' : // PAUSE
			infoRightShow( $TASKLIST.SELECTED );
			$( '#remove-link' ).removeClass( 'disabled' );
			$( '#resume-link' ).removeClass( 'disabled' );
			break;
		case '5' : // COMPLETE
			infoRightShow( $TASKLIST.SELECTED );
			$( '#remove-link' ).removeClass( 'disabled' );
			break;
		case '6' : // FAILURE
			infoRightShow( $TASKLIST.SELECTED );
			$( '#remove-link' ).removeClass( 'disabled' );
			$( '#resume-link' ).removeClass( 'disabled' );
			break;
	}
}

// 全局点击事件
function globalClickEvents() {

	// 取消select展开
	if( $SELECT.MOUSEON == false && $SELECT.OPEN == true && $SELECT.OBJ != null ) {
		$SELECT.OBJ.height( 16 );
		$SELECT.OBJ = null;
		$SELECT.OPEN = false;
	}

	// 点击了非任务列，取消选择
	if( $TASKLIST.MOUSEON == false  && $INFO_RIGHT.MOUSEON == false ) {
		$TASKLIST.SELECTED = null;
		$( '#Task-list div[task_id]' ).removeClass( 'selected' );
		$( '#remove-link' ).addClass( 'disabled' );
		$( '#resume-link' ).addClass( 'disabled' );
		$( '#pause-link' ).addClass( 'disabled' );
		// infoRight
		infoRightHide();
		// 不响应Disabled状态的按钮事件
		if( ( $( this ).attr( 'class' ) != undefined && $( this ).attr( 'class' ).indexOf( 'disabled' ) > -1 ) ) return false;
	}

}

// 刷新任务状态及显示
function displayUpdateInterval() {
	$.get( 'main.php?action=select&rand=' + Math.random(), function(x) {
		if( x != null ) {
			$TASK.DATA = null;
			$TASK.DATA = x;
		} else {
			$TASK.DATA = null;
		}
		display();
		setTimeout( displayUpdateInterval, $SETTING.REFRESH );
	}, 'json' )
}

// 显示刷新
function display() {

	$( '#Task-list div[id]:gt(0)' ).html( '' );
	// INIT REMOVE
	$( '#Task-list div[id]:eq(0) [status=1]' ).remove();
	$( '#pause-all-link' ).removeClass().addClass( 'disabled' );
	$( '#resume-all-link' ).removeClass().addClass( 'disabled' );
	if( $TASK.DATA == null || $TASK.DATA.length < 1 ) return false;

	var pause_enabled = false;
	var resume_enabled = false;

	for( var i = 0; i < $TASK.DATA.length; i ++ ) {
		if( false == pause_enabled && $TASK.DATA[ i ].status <= 3 ) {
			$( '#pause-all-link' ).removeClass();
			pause_enabled = true;
		} else if( false == resume_enabled && $TASK.DATA[ i ].status == 4 ) {
			$( '#resume-all-link' ).removeClass();
			resume_enabled = true;
		}
		if( $TASK.ACTIVE[ $TASK.DATA[ i ].id ] != undefined && ( $TASK.DATA[ i ].status != 1 && $TASK.DATA[ i ].status != 3 ) ) {
			clearActive( $TASK.DATA[ i ].id );
		}
		appendTask( $TASK.DATA[ i ] );
	}

	return false;

	/* PRIVATE FUNCTION */

	/* 任务信息显示 */
	function appendTask( info ) {

		var output = '', ps = '', tc = '';
		switch( info.status ) {
			// 初始化的任务也进入start-list但不创建活动事件
			case '1' : ps = 'left bottom'; output = 'start-list'; tc='初始化'; break;
			case '2' : ps = 'left -24px'; output = 'standby-list'; tc='队列等待'; break;
			case '3' : ps = 'left top'; output = 'start-list'; tc='下载中'; break;
			case '4' : ps = 'left -48px'; output = 'pause-list'; tc='任务暂停'; break;
			case '5' : ps = 'left -72px'; output = 'complete-list'; tc='下载完成'; break;
			case '6' : ps = 'left -96px'; output = 'failure-list'; tc='下载失败'; break;
		}

		var rate = rateCount( info.data, info.size );
		var progress = round( 116 * ( rate / 100 ) );
		var fcolor = rate >= 65 ? 'white' : 'black';
		// 更新时，也保持选中状态显示
		var bg = $TASKLIST.SELECTED == info.id ? 'class="list selected"' : 'class="list"';

		// 活动的任务
		if( info.status == '3' ) {
			if( $TASK.ACTIVE[ info.id ] == undefined ) {
				newActive( info, bg, progress, rate, fcolor, tc );
			} else {
				updateActive( info, bg, progress, rate, fcolor );
			}
		// 非活动任务
		} else {
			if( $TASK.LAST_FINISH[ info.id ] != undefined ) {
				progress = $TASK.LAST_FINISH[ info.id ].PROGRESS;
				rate = $TASK.LAST_FINISH[ info.id ].RATE;
			}
			var completeTime = '';
			var completeTc = '';
			// 已经完成的任务，显示完成时间
			if( info.status == 5 && info.update_time != 0 ) {
				completeTime = $.dateFormat( info.update_time, 'YYYY-MM-dd hh:mm:ss' );
				completeTc = '创建时间: ' + $.dateFormat( info.create_time, 'YYYY-MM-dd hh:mm:ss' ) + ', 完成时间: ' + $.dateFormat( info.update_time, 'YYYY-MM-dd hh:mm:ss' );
			}

			$( '#' + output ).append( '<div task_id="'+ info.id +'" status="'+ info.status +'" '+bg+'><div class="status" style="background-position:'+ ps +';" titleContents="'+ tc +'"></div><div class="info"><div class="name">'+info.name+'</div><div class="size">'+ sizeCount( info.size ) +'</div></div><div class="progress-block"><div class="progress"><div class="bg" style="width:'+ progress +'px;"></div><div class="rate" style="color:' + fcolor + ';">'+ rate +' %</div></div><div class="progress-size">'+ sizeCount( info.data ) +' / '+ sizeCount( info.size ) +'</div></div><div class="times"><div class="runtime">'+ timeCount( info.runtime ) +'</div><div class="speed"></div></div><div class="remaining"><div class="remaining-time"></div><div class="complete-time" titleContents="'+ completeTc +'">'+ completeTime +'</div></div>' );
			// 最后完结动画
			if( $TASK.LAST_FINISH[ info.id ] != undefined ) {
				delete $TASK.LAST_FINISH[ info.id ];
				$( '#' + output + ' [task_id=' + info.id + '] .bg' ).animate( { 'width' : 116 }, 1200 );
				numplus( rate, 100, $( '#' + output + ' [task_id=' + info.id + '] .rate' ), ' %' );
			}
		}

		return false;
	}

	/* 新建活动的任务 */
	function newActive( info, bg, progress, rate, fcolor, tc ) {
		$( '#start-list' ).html( '' );
		$TASK.ACTIVE[ info.id ] = {};
		// 记录 timeStamp 用来统计计算
		$TASK.ACTIVE[ info.id ].TIME_STAMP = timeStamp();
		$TASK.ACTIVE[ info.id ].BYTES = info.data;
		$TASK.ACTIVE[ info.id ].RUNTIME = parseInt( info.runtime );
		$TASK.ACTIVE[ info.id ].RATE = rate;
		var speed = sizeCount( $TASK.ACTIVE[ info.id ].BYTES / $SETTING.REFRESH );
		$( '#start-list' ).append( '<div task_id="'+ info.id +'" status="'+ info.status +'" '+bg+'><div class="status" style="background-position:left top;" titleContents="'+ tc +'"></div><div class="info"><div class="name">'+info.name+'</div><div class="size">'+ sizeCount( info.size ) +'</div></div><div class="progress-block"><div class="progress"><div class="bg" style="width:'+ progress +'px;"></div><div class="rate" style="color:' + fcolor + ';">'+ rate +' %</div></div><div class="progress-size">'+ sizeCount( info.data ) +' / '+ sizeCount( info.size ) +'</div></div><div class="times"><div class="runtime">'+ timeCount( info.runtime ) +'</div><div class="speed">' + speed + ' /s</div></div><div class="remaining"><div class="remaining-time">??:??:??</div><div class="complete-time"></div></div>' );

		// 添加一个自增计时
		$TASK.ACTIVE[ info.id ].INTERVAL_ID = setInterval( function() {
			$TASK.ACTIVE[ info.id ].RUNTIME ++;
			$( '#start-list [task_id=' + info.id + '] [class=runtime]' ).html( timeCount( $TASK.ACTIVE[ info.id ].RUNTIME ) );
		}, 1000 );

	}

	/* 更新活动任务 */
	function updateActive( info, bg, progress, rate, fcolor ) {

		if( $TASK.ACTIVE[ info.id ] == undefined ) return false;

		// 本次数据对比上次的差值
		var differ = info.data - $TASK.ACTIVE[ info.id ].BYTES;
		// 上一次更新到现在经过的毫秒数
		var interval = timeStamp() - $TASK.ACTIVE[ info.id ].TIME_STAMP;
		var sec = Math.floor( interval / 1000 );
		// 更新即时速度&预计剩余时间
		if( differ > 0 ) {
			var speed = sizeCount( round( differ / sec, 2 ) ) + '/s';
			$( '#start-list div[task_id=' + info.id + '] .speed' ).html( speed );
			var remaining = timeCount( round( ( info.size - info.data ) / differ ) );
			$( '#start-list div[task_id=' + info.id + '] .remaining-time' ).html( remaining );
		} else {
			$( '#start-list div[task_id=' + info.id + '] .speed' ).html( 'N/A' );
			$( '#start-list div[task_id=' + info.id + '] .remaining-time' ).html( '??:??:??' );
		}
		// 更新实际完成量
		$( '#start-list [task_id=' + info.id + '] .progress-size' ).text( sizeCount( info.data ) + ' / ' + sizeCount( info.size ) );
		// 上一次的长度
		var last_progress = $( '#start-list [task_id=' + info.id + '] .bg' ).width();
		// 动画效果
		if( progress > last_progress ) {
			$TASK.ACTIVE[ info.id ].PROGRESS = progress;
			$( '#start-list [task_id=' + info.id + '] .bg' ).animate( { 'width' : progress }, round( interval / 3 ) );
			numplus( $TASK.ACTIVE[ info.id ].RATE, rate, $( '#start-list [task_id=' + info.id + '] .rate' ), ' %' );
			$( '#start-list [task_id=' + info.id + '] .rate' ).css( 'color', fcolor );
		}
		// 记录进度
		$TASK.ACTIVE[ info.id ].RATE = rate;
		$TASK.ACTIVE[ info.id ].TIME_STAMP = timeStamp();
		$TASK.ACTIVE[ info.id ].BYTES = info.data;

	}

} // display end

// 删除一个活动中的任务显示及事件
function clearActive( id ) {
	if( $TASK.ACTIVE[ id ] == undefined ) return false;
	clearInterval( $TASK.ACTIVE[ id ].INTERVAL_ID );
	$( '#start-list [task_id=' + id + ']' ).remove();
	var info = getTask( id );
	if( info != null && info.size == info.data ) {
		$TASK.LAST_FINISH[ id ] = {
			PROGRESS : $TASK.ACTIVE[ id ].PROGRESS,
			RATE : $TASK.ACTIVE[ id ].RATE
		};
	}
	delete $TASK.ACTIVE[ id ];
}

// 获取匹配ID的任务
function getTask( id ) {
	if( $TASK.DATA == null || $TASK.DATA.length < 1 ) return null;
	for( var i = 0; i < $TASK.DATA.length; i ++ ) {
		if( $TASK.DATA[ i ].id == id ) return $TASK.DATA[ i ];
	}
	return null;
}