/**
 * MovePlus For jQuery
 * Version 1.3
 *
 * @Application : jquery.moveplus.js
 * @Encode : 65001 Unicode ( UTF-8 NO BOM )
 * @Description : 拖动插件强化版
 * @Author : HK <hkyyang@qq.com>
 * @Update : 2010-11-11
 */

( function( $ ) {

	var __HK_MOVE_PLUS_MOUSEDOWN__ = null;
	var __HK_MOVE_PLUS_MAIN__ = {};
	var __HK_MOVE_PLUS_MAIN_COPY__ = {};
	var __HK_MOVE_PLUS_HANDLER__ = {};
	var __HK_MOVE_PLUS_STATUS__ = {};
	var __HK_MOVE_PLUS_LIMIT__ = {};
	var __HK_MOVE_PLUS_CALLBACK__ = {};

	var __HK_MOVE_PLUS_LAST_X__ = __HK_MOVE_PLUS_LAST_Y__ = null;
	var __HK_MOVE_PLUS_LAST_LEFT__ = __HK_MOVE_PLUS_LAST_TOP__ = null;

	var __HK_MOVE_PLUS_DEBUG__ = null;

	/*------------------------------------ Public ------------------------------------*/

	/**
	 * 对象拖动
	 *
	 * @param String | Object | Null, $handler 移动对象中可触发拖动的部分,一般为handler, 当为Null时, 为移动对象本体
	 * @param String | Object | Null, $parent 父对象, 当不为Null时, 子对象只能移动在父对象中.默认为document
	 * @param Object | Null, $callback 回调函数, DOWN:mousedown, UP:mouseup, MOVE:mousemove
	 */
	$.fn.moveOn = function( $handler, $parent, $callback ) {

		return $( this ).each( function( i ) {

			var HANDLER_OBJ = $( this ).moveSetting( $handler, $parent );
			var HANDLER_ID = HANDLER_OBJ.attr( 'id' );
			__HK_MOVE_PLUS_CALLBACK__[ HANDLER_ID ] = $callback ? $callback : null;
			__HK_MOVE_PLUS_MAIN__[ HANDLER_ID ] = $( this );
			__HK_MOVE_PLUS_HANDLER__[ HANDLER_ID ] = HANDLER_OBJ;

			HANDLER_OBJ.bind( 'mousedown', function( e ) {

				$( this ).css( 'cursor', 'move' );
				__HK_MOVE_PLUS_MAIN__[ $( this ).attr( 'id' ) ].css( 'z-index', parseInt( new Date().getTime() / 1000 ) );

				var MAIN_OFFSET = __HK_MOVE_PLUS_MAIN__[ this.id ].offset();
				var PAGE = $.moveGetMP( e );
				__HK_MOVE_PLUS_LAST_X__ = PAGE.X;
				__HK_MOVE_PLUS_LAST_Y__ = PAGE.Y;
				__HK_MOVE_PLUS_LAST_LEFT__ = MAIN_OFFSET.left;
				__HK_MOVE_PLUS_LAST_TOP__ = MAIN_OFFSET.top;
				__HK_MOVE_PLUS_MOUSEDOWN__ = this.id;

				if( __HK_MOVE_PLUS_CALLBACK__[ this.id ] != null && __HK_MOVE_PLUS_CALLBACK__[ this.id ].DOWN != null  ) {
					__HK_MOVE_PLUS_CALLBACK__[ this.id ].DOWN( e, __HK_MOVE_PLUS_MAIN__[ this.id ] );
				}

				$.moveUpdate( e );

				return false;

			} );

		} );

	};

	/**
	 * 对象拖动, 镜像模式, 生成对象副本, 拖动副本移动实体
	 *
	 * @param String | Object | Null, $handler 移动对象中可触发拖动的部分,一般为handler, 当为Null时, 为移动对象本体
	 * @param String | Object | Null, $parent 父对象, 当不为Null时, 子对象只能移动在父对象中.默认为document
	 * @param Object | Null, $callback 回调函数, DOWN:mousedown, UP:mouseup, MOVE:mousemove
	 */
	$.fn.moveCopy = function( $handler, $parent, $callback ) {

		return $( this ).each( function() {
			var HANDLER_OBJ = $( this ).moveSetting( $handler, $parent );
			var HANDLER_ID = HANDLER_OBJ.attr( 'id' );
			__HK_MOVE_PLUS_CALLBACK__[ HANDLER_ID ] = $callback ? $callback : null;

			__HK_MOVE_PLUS_HANDLER__[ HANDLER_ID ] = HANDLER_OBJ;
			__HK_MOVE_PLUS_MAIN_COPY__[ HANDLER_ID ] = $( this );

			HANDLER_OBJ.bind( 'mousedown', function( e ) {
				$( this ).css( 'cursor', 'move' );
				__HK_MOVE_PLUS_MAIN__[ $( this ).attr( 'id' ) ].css( 'z-index', parseInt( new Date().getTime() / 1000 ) );
				var PAGE = $.moveGetMP( e );
				var COPY_OBJ = __HK_MOVE_PLUS_MAIN_COPY__[ this.id ].clone( true );
				var COPY_X = PAGE.X - Math.floor( $( this ).width() / 2 );
				var COPY_Y = PAGE.Y - Math.floor( $( this ).height() / 2 );
				COPY_OBJ.appendTo( 'body' ).css( { 'left' : COPY_X, 'top' : COPY_Y, 'opacity' : 0.5 } );

				__HK_MOVE_PLUS_MAIN__[ this.id ] = COPY_OBJ;

				var MAIN_OFFSET = __HK_MOVE_PLUS_MAIN__[ this.id ].offset();
				__HK_MOVE_PLUS_LAST_X__ = PAGE.X;
				__HK_MOVE_PLUS_LAST_Y__ = PAGE.Y;
				__HK_MOVE_PLUS_LAST_LEFT__ = MAIN_OFFSET.left;
				__HK_MOVE_PLUS_LAST_TOP__ = MAIN_OFFSET.top;
				__HK_MOVE_PLUS_MOUSEDOWN__ = this.id;

				if( __HK_MOVE_PLUS_CALLBACK__[ this.id ] != null && __HK_MOVE_PLUS_CALLBACK__[ this.id ].DOWN != null  ) {
					__HK_MOVE_PLUS_CALLBACK__[ this.id ].DOWN( e, __HK_MOVE_PLUS_MAIN__[ this.id ] );
				}

				$.moveUpdate( e );

				return false;

			} );

		} );

	};

	/**
	 * 关闭对象拖动
	 *
	 * @public
	 *
	 * @param String | Object | Null, $handler 移动对象中可触发拖动的部分,一般为handler, 当为Null时, 为移动对象本体
	 * @return Boolean
	 */
	$.fn.moveOff = function( $handler ) {

		return $( this ).each( function() {

			var HANDLER_OBJ = null;
			var HANDLER_ID = HANDLER_OBJ.attr( 'id' );

			if( $handler == null ) {
				HANDLER_OBJ = $( this );
			} else if( typeof $handler == 'string' ) {
				HANDLER_OBJ = $( '#' + $handler );
			} else {
				HANDLER_OBJ = $handler;
			}
			__HK_MOVE_PLUS_STATUS__[ HANDLER_ID ] = false;

			return false;

		} );

	};

	// Animation 移动到坐标
	$.fn.moveTo = function( x, y, speed, callback ) {

		return $( this ).each( function() {
			var offset = $( this ).offset();
			if( speed == null || speed < 1 || ( offset.left == x && offset.left == y ) ) {
				$( this ).css( { 'left' : x + 'px', 'top' : y + 'px' } );
				callback();
				return;
			}
			if( $.isFunction( callback ) ) {
				$( this ).animate( { 'left' : x + 'px', 'top' : y + 'px' }, speed, callback );
			} else {
				$( this ).animate( { 'left' : x + 'px', 'top' : y + 'px' }, speed );
			}
		} );

	};

	/**
	 * 用于调试
	 *
	 * @public
	 *
	 * @param Boolean, $switch 显示调试开关
	 * @param String | Object | Null, $out 指定输出对象
	 * @param Boolean, $autohide 只在事件过程中显示调试信息
	 * @return Boolean
	 */
	$.moveDebug = function( $switch, $out, $autohide ) {

		var OUT_OBJ = null;
		if( $out == null ) {
			OUT_OBJ = null;
		} else if( typeof $out == 'string' ) {
			OUT_OBJ = $( '#' + $out );
		} else {
			OUT_OBJ = $out;
		}

		__HK_MOVE_PLUS_DEBUG__ = {
			'SWITCH' : $switch ? true : false,
			'OUTPUT' : OUT_OBJ,
			'AUTOHIDE' : $autohide ? true : false
		};

		return false;

	};

	/**
	 * 对象水平垂直居中,如果要指定参照父对象, 要确保大小不大于父对象
	 * 注意, 只适用于绝对定位, 对象position属性会改变
	 *
	 * @public
	 *
	 * @param String | Object | Null, $parent 指定参照父对象( 相对父对象居中 )
	 * @param Boolean, $return 只返回计算结果
	 * @return Boolean | Object
	 */
	$.fn.moveCenter = function( $parent, $return ) {

			var THIS_OBJ = $( this );

			// 父对象设置
			var PARENT_OBJ = PARENT_W = PARENT_H = PARENT_OFFSET = null;
			var SCROLL_TOP = SCROLL_LEFT = 0;
			if( $parent == null ) {
				PARENT_OBJ = $( self.document );
			} else if( typeof $parent == 'string' ) {
				PARENT_OBJ = $( '#' + $parent );
			} else {
				PARENT_OBJ = $( $parent );
			}

			PARENT_OFFSET = PARENT_OBJ.offset();
			PARENT_W = PARENT_OBJ.width();
			PARENT_H = PARENT_OBJ.height();
			SCROLL_LEFT = PARENT_OBJ.scrollLeft();
			SCROLL_TOP = PARENT_OBJ.scrollTop();
			SCROLL_LEFT = SCROLL_LEFT > 0 ? SCROLL_LEFT / 2 : 0;
			SCROLL_TOP = SCROLL_TOP > 0 ? SCROLL_TOP / 2 : 0;

			// 位置计算
			var AVAIL_H = window.screen.availHeight;
			var AVAIL_W = window.screen.availWitch;
			var WINDOW_H = $( self.document ).height();
			var WINDOW_W = $( self.document ).width();

			var ADDED_H = ADDED_W = 0;
			if( AVAIL_H < WINDOW_H ) {
				ADDED_H = ( WINDOW_H - AVAIL_H ) / 2;
			}
			if( AVAIL_W < WINDOW_W ) {
				ADDED_W = ( WINDOW_W - AVAIL_W ) / 2;
			}
			var THIS_W = THIS_OBJ.width();
			var THIS_H = THIS_OBJ.height();
			var X = parseInt( PARENT_W / 2 - THIS_W / 2 ) - ADDED_W + SCROLL_LEFT;
			var Y = parseInt( PARENT_H / 2 - THIS_H / 2 )   - ADDED_H + SCROLL_TOP;

			if( PARENT_OFFSET != null ) {
				X += PARENT_OFFSET.left;
				Y += PARENT_OFFSET.top;
			}

			if( $return == true ) return { 'left' : X, 'top' : Y };

			THIS_OBJ.css( {
				'position' : 'absolute',
				'left' : X,
				'top' : Y
			} );

			return $( this );

	};

	/*------------------------------------ Private ------------------------------------*/

	/**
	 * 范围限制等设定
	 *
	 * @private
	 *
	 * @param String | Object | Null, $handler 移动对象中可触发拖动的部分,一般为handler, 当为Null时, 为移动对象本体
	 * @param String | Object | Null, $parent 父对象, 当不为Null时, 子对象只能移动在父对象中.默认为document
	 * @return Object
	 */
	$.fn.moveSetting = function( $handler, $parent ) {

		var HANDLER_OBJ = HANDLER_ID = null;
		var PARENT_OBJ = null;

		// HANDLER
		if( $handler == null ) {
			HANDLER_OBJ = $( this );
		} else if( typeof $handler == 'string' ) {
			HANDLER_OBJ = $( '#' + $handler );
		} else {
			HANDLER_OBJ = $handler;
		}
		if( HANDLER_OBJ.attr( 'id' ) == null ) {
			HANDLER_OBJ.attr( 'id', 'MOVE_PLUS_HANDLER_' + ( new Date().getTime() ) );
		}
		HANDLER_ID = HANDLER_OBJ.attr( 'id' );

		__HK_MOVE_PLUS_MAIN__[ HANDLER_ID ] = $( this );
		__HK_MOVE_PLUS_STATUS__[ HANDLER_ID ] = true;

		// 父对象设置
		if( $parent == null ) {
			PARENT_OBJ = $( document );
		} else if( typeof $parent == 'string' ) {
			PARENT_OBJ = $( '#' + $parent );
		} else {
			PARENT_OBJ = $parent;
		}

		// 移动范围限制
		if( PARENT_OBJ != null ) {
			var PARENT_OFFSET = PARENT_OBJ.offset();
			__HK_MOVE_PLUS_LIMIT__[ HANDLER_ID ] = {
				'left' :	 PARENT_OFFSET != null ? PARENT_OFFSET.left : 0,
				'top' : PARENT_OFFSET != null ? PARENT_OFFSET.top : 0,
				'right' : PARENT_OFFSET != null ? PARENT_OFFSET.left + PARENT_OBJ.width() : PARENT_OBJ.width(),
				'bottom' : PARENT_OFFSET != null ? PARENT_OFFSET.top + PARENT_OBJ.height() : PARENT_OBJ.height()
			};
		}

		return HANDLER_OBJ;

	};

	// 鼠标按键弹起事件
	$( document ).bind( 'mouseup', function( e ) {

		if( __HK_MOVE_PLUS_MOUSEDOWN__ != null && __HK_MOVE_PLUS_STATUS__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] == true ) {

			__HK_MOVE_PLUS_HANDLER__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].css( 'cursor', 'default' );
			// 回调事件
			if( __HK_MOVE_PLUS_CALLBACK__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] != null && __HK_MOVE_PLUS_CALLBACK__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].UP != null  ) {
				__HK_MOVE_PLUS_CALLBACK__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].UP( e, __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] );
			}
			// 调试模式, 无事件自动隐藏调试信息
			if( __HK_MOVE_PLUS_DEBUG__ != null ) {
				if( __HK_MOVE_PLUS_DEBUG__.SWITCH == true && __HK_MOVE_PLUS_DEBUG__.AUTOHIDE == true ) {
					__HK_MOVE_PLUS_DEBUG__.OUTPUT.hide();
				}
			}
			// 镜像模式:副本再次复制成副本
			if( __HK_MOVE_PLUS_MAIN_COPY__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] != null ) {
				__HK_MOVE_PLUS_MAIN_COPY__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].remove();
				__HK_MOVE_PLUS_MAIN_COPY__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] = __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ];
				__HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].css( { 'cursor':'default', 'opacity':1 } ).find( '#' + __HK_MOVE_PLUS_HANDLER__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].attr( 'id' ) ).css( 'cursor', 'default' ) ;
			}

			__HK_MOVE_PLUS_MOUSEDOWN__ = null;

			return false;
		}

	} );

	// 鼠标移动事件
	$( document ).bind( 'mousemove', function( e ) {

		if( __HK_MOVE_PLUS_MOUSEDOWN__ != null && __HK_MOVE_PLUS_STATUS__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] == true ) {

			$.moveUpdate( e );

			// 移动事件
			if( __HK_MOVE_PLUS_CALLBACK__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] != null && __HK_MOVE_PLUS_CALLBACK__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].MOVE != null  ) {
				__HK_MOVE_PLUS_CALLBACK__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].MOVE( { 'X' : $POSITION.X, 'Y':$POSITION.Y }, __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] );
			}

			return false;

		}

	} );

	/**
	 * 计算对象位置
	 *
	 * @private
	 *
	 * @param e
	 * @return Object
	 */
	$.moveGetPosition = function( e ) {

		var PAGE = $.moveGetMP( e );

		var POSITION_X = PAGE.X - __HK_MOVE_PLUS_LAST_X__ + __HK_MOVE_PLUS_LAST_LEFT__;
		var POSITION_Y = PAGE.Y - __HK_MOVE_PLUS_LAST_Y__ + __HK_MOVE_PLUS_LAST_TOP__;

		// Limit
		if( __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ] != null ) {

			if( POSITION_X < __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].left ) {
				POSITION_X = __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].left;
			} else if( ( POSITION_X + __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].width()  ) > __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].right ) {
				POSITION_X = __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].right - __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].width();
			}

			if( POSITION_Y < __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].top ) {
				POSITION_Y = __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].top;
			} else if( ( POSITION_Y + __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].height() ) > __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].bottom ) {
				POSITION_Y = __HK_MOVE_PLUS_LIMIT__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].bottom - __HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].height();
			}

		}

		return { 'X' : POSITION_X, 'Y' : POSITION_Y };
	};

	/**
	 * 获取鼠标指针当前偏移坐标
	 *
	 * @private
	 *
	 * @param e
	 * @return Object
	 */
	$.moveGetMP = function( e ) {

		var PAGE_X = PAGE_Y = 0;

		if ( ! e ) e = window.event;

		if ( e.pageX || e.pageY ) {
			PAGE_X = e.pageX;
			PAGE_Y = e.pageY;
		} else if ( e.clientX || e.clientY ) {
			PAGE_X = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
			PAGE_Y = e.clientY + document.body.scrollTop  + document.documentElement.scrollTop;
		}

		return { 'X': PAGE_X, 'Y': PAGE_Y };
	};

	/**
	 * 更新主对象位置
	 *
	 * @private
	 *
	 * @param e
	 * @return boolean
	 */
	$.moveUpdate = function( e ) {

		var $POSITION = $.moveGetPosition( e );

		__HK_MOVE_PLUS_MAIN__[ __HK_MOVE_PLUS_MOUSEDOWN__ ].css( {
			'left' : $POSITION.X,
			'top' : $POSITION.Y
		} );

		if( __HK_MOVE_PLUS_DEBUG__ != null && __HK_MOVE_PLUS_DEBUG__ .SWITCH == true ) {

			if( __HK_MOVE_PLUS_DEBUG__.OUTPUT == null ) {
				$( 'body' ).append( '<div id="MOVE_PLUS_DEBUG" style="position:absolute; right:0; bottom:0; padding:0 10px 0 10px; text-align:center; height:20px; line-height:20px; color:#999; border:1px solid #CCC; z-index:999; font-size:12px; "></div>' );
				__HK_MOVE_PLUS_DEBUG__.OUTPUT = $( '#MOVE_PLUS_DEBUG' );
			}
			__HK_MOVE_PLUS_DEBUG__.OUTPUT.html( 'X:' + $POSITION.X + ' Y:' + $POSITION.Y + ' OBJ:' + __HK_MOVE_PLUS_MOUSEDOWN__ );
			if( __HK_MOVE_PLUS_DEBUG__.AUTOHIDE == true ) {
				__HK_MOVE_PLUS_DEBUG__.OUTPUT.show();
			}
		}

	};

} ) ( jQuery );